#!/usr/local/bin/ruby -w
#
# vim:sw=2 ts=8:et sta:fdm=marker
#
#
# Copyright (c) 2008 Ariff Abdullah <ariff@FreeBSD.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#        $FreeBSD$
#

def vseg(v, o)
  r = o
  v >>= 7
  v >>= o
  if (v & 0xf0) != 0
    v >>= 4
    r += 4
  end
  if (v & 0x0c) != 0
    v >>= 2
    r += 2
  end
  if (v & 0x02) != 0
    r += 1
  end
  r
end

def alaw_to_u8(v)
  v ^= 0x55
  t = v & 0x7f
  if t < 0x10
    t = (t << 4) + 8
  else
    s = (t >> 4) & 0x07
    t = ((t & 0x0f) << 4) + 0x108
    t <<= s - 1
  end

  v = (((v & 0x80) != 0) ? t : -t) >> 8
  if v < 0
    v += 0x100
  end

  v ^ 0x80
end

def u8_to_alaw(v)
  v ^= 0x80
  if v > 0x7f
    v = v - 0x100
  end
  v <<= 8
  if v >= 0
    m = 0xd5
  else
    m = 0x55
    v = -v
    if v > 0x7fff
      v = 0x7fff
    end
  end

  if v < 0x100
    a = v >> 4
  else
    s = vseg(v, 1)
    a = (s << 4) | ((v >> (s + 3)) & 0x0f)
  end

  a ^ m
end

def ulaw_to_u8(v)
  v = ~v
  t = ((v & 0x0f) << 3) + 0x84
  t <<= (v & 0x70) >> 4

  v = (((v & 0x80) != 0) ? (0x80 - t) : (t - 0x84)) >> 8
  if v < 0
    v += 0x100
  end

  v ^ 0x80
end

def u8_to_ulaw(v)
  v ^= 0x80
  if v > 0x7f
    v = v - 0x100
  end
  v <<= 8
  if v < 0
    v = 0x84 - v
    m = 0x7f
  else
    v += 0x84
    m = 0xff
  end
  
  if v > 0x7fff
    v = 0x7fff
  end

  s = vseg(v, 0)
  u = (s << 4) | ((v >> (s + 3)) & 0x0f)

  u ^ m
end

def gentab(t, f)
  printf("#define %s\t{\t\t\t\t\t\t\\\n", t)
  0.upto(255) do |c|
    if c % 8 == 0
      printf("\n") unless c == 0
      printf("\t")
    end
    printf("%5d,", f.call(c))
    printf("\t\t\\") if (c + 1) % 8 == 0
  end
  printf("\n\t}\n")
end

printf("/*\n")
printf(" * THIS FILE IS GENERATED. DO NOT EDIT!\n")
printf(" */\n\n")
printf("#ifndef __SND_G711_H__\n")
printf("#define __SND_G711_H__\n\n")
[
  ['ULAW_TO_U8', method(:ulaw_to_u8)],
  ['ALAW_TO_U8', method(:alaw_to_u8)],
  ['U8_TO_ULAW', method(:u8_to_ulaw)],
  ['U8_TO_ALAW', method(:u8_to_alaw)],
].each do |t, f|
  gentab(t, f)
  printf("\n")
end
printf("#endif\t/* __SND_G711_H__ */\n")
